/*
-----------------------------------------------------------------------------
-      ATMEL Microcontroller Software Support  -   ROUSSET -
-----------------------------------------------------------------------------
 The software is delivered "AS IS" without warranty or condition of any
 kind, either express, implied or statutory. This includes without
 limitation any warranty or condition with respect to merchantability or
 fitness for any particular purpose, or against the infringements of
 intellectual property rights of others.
-----------------------------------------------------------------------------
- File Name            : pio.inc
- Object               : PIO Controller Definition File.
- Translator           : ARM Software Development Toolkit V2.11a
-
- 1.0 10/03/98 JCZ     : Creation
- 2.0 21/10/98 JCZ     : Clean up.
-----------------------------------------------------------------------------
*/
/*
---------------------------------
 Parallel IO Controller Structure
---------------------------------
*/
.equ PIO_PER,0x00       /* PIO Enable Register */
.equ PIO_PDR,0x04       /* PIO Disable Register */
.equ PIO_PSR,0x08       /* PIO Status Register */

.equ PIO_OER,0x10       /* Output Enable Register */
.equ PIO_ODR,0x14       /* Output Disable Register */
.equ PIO_OSR,0x18       /* Output Status Register */

.equ PIO_IFER,0x20       /* Input Filter Enable Register */
.equ PIO_IFDR,0x24       /* Input Filter Disable Register */
.equ PIO_IFSR,0x28       /* Input Filter Status Register */

.equ PIO_SODR,0x30       /* Set Output Data Register */
.equ PIO_CODR,0x34       /* Clear Output Data Register */
.equ PIO_ODSR,0x38       /* Output Data Status Register */
.equ PIO_PDSR,0x3c       /* Pin Data Status Register */
.equ PIO_IER,0x40       /* Interrupt Enable Register */
.equ PIO_IDR,0x44       /* Interrupt Disable Register */
.equ PIO_IMR,0x48       /* Interrupt Mask Register */
.equ PIO_ISR,0x4c       /* Interrupt Status Register */

/*
--------------------------------
- Device Dependancies Definition
--------------------------------
*/
    .ifdef AT91M40400

/*- Number of PIO Controller */
.equ NB_PIO_CTRL,1

/*- Base Address */
.equ PIO_BASE,0xFFFF0000

/*- Number of PIO Lines */
.equ NB_PIO,32

/*- Parallel I/O Bits Definition */
.equ P0,(1<<0)
.equ P1,(1<<1)
.equ P2,(1<<2)
.equ P3,(1<<3)
.equ P4,(1<<4)
.equ P5,(1<<5)
.equ P6,(1<<6)
.equ P7,(1<<7)
.equ P8,(1<<8)
.equ P9,(1<<9)
.equ P10,(1<<10)
.equ P11,(1<<11)
.equ P12,(1<<12)
.equ P13,(1<<13)
.equ P14,(1<<14)
.equ P15,(1<<15)
.equ P16,(1<<16)
.equ P17,(1<<17)
.equ P18,(1<<18)
.equ P19,(1<<19)
.equ P20,(1<<20)
.equ P21,(1<<21)
.equ P22,(1<<22)
.equ P23,(1<<23)
.equ P24,(1<<24)
.equ P25,(1<<25)
.equ P26,(1<<26)
.equ P27,(1<<27)
.equ P28,(1<<28)
.equ P29,(1<<29)
.equ P30,(1<<30)
.equ P31,(1<<31)

/*- PIO Multiplexing Definition */

.equ PIO_CTRL,0

.equ TCLK0,P0
.equ TIOA0,P1
.equ TIOB0,P2
.equ PIN_TC0,TIOA0 | TIOB0 | TCLK0

.equ TCLK1,P3
.equ TIOA1,P4
.equ TIOB1,P5
.equ PIN_TC1,TIOA1 | TIOB1 | TCLK1

.equ TCLK2,P6
.equ TIOA2,P7
.equ TIOB2,P8
.equ PIN_TC2,TIOA2 | TIOB2 | TCLK2

.equ PIN_IRQ0,P9
.equ PIN_IRQ1,P10
.equ PIN_IRQ2,P11
.equ PIN_FIQ,P12

.equ PIO_USART0,0
.equ SCK0,P13
.equ TXD0,P14
.equ RXD0,P15
.equ PIN_USART0,SCK0 | TXD0 | RXD0

.equ PIO_USART1,0
.equ SCK1,P20
.equ TXD1,P21
.equ RXD1,P22
.equ PIN_USART1,SCK1 | TXD1 | RXD1

.equ MCKO,P25
.equ CS2,P26
.equ CS3,P27
.equ CS4,P31
.equ CS5,P30
.equ CS6,P29
.equ CS7,P28

    .endif   /* AT91M40400 */


